from http import client
import imp
import pytest
from arkindex_worker.worker.training import create_archive, TrainingMixin
import os
import responses
from responses import matchers
from pathlib import Path

from arkindex.mock import MockApiClient

def test_create_archive_folder():
    model_file_dir = Path("tests/samples/model_files")

    with create_archive(path=model_file_dir) as (
        zst_archive_path,
        hash,
        size,
        archive_hash,
    ):
        assert os.path.exists(zst_archive_path), "The archive was not created"
        assert (
            hash == "7dd70931222ef0496ea75e5aee674043"
        ), "Hash was not properly computed"
        assert 300 < size < 700

    assert not os.path.exists(zst_archive_path), "Auto removal failed"


def test_create_model_version():
    api_client = MockApiClient()

    """A new model version is returned"""
    model_id = "fake_model_id"
    model_version_id = "fake_model_version_id"
    # Create a model archive and keep its hash and size.
    model_files_dir = Path("tests/samples/model_files")
    # model_file_path = model_files_dir / "model_file.pth"
    training = TrainingMixin()
    with create_archive(path=model_files_dir) as (
        zst_archive_path,
        hash,
        size,
        archive_hash,
    ):
        model_version_details = {
            "id": model_version_id,
            "model_id": model_id,
            "hash": hash,
            "archive_hash": archive_hash,
            "size": size,
            "s3_url": "http://hehehe.com",
            "s3_put_url": "http://hehehe.com",
        }

    responses.add(
        responses.POST,
        f"http://testserver/api/v1/model/{model_id}/versions/",
        status=200,
        match=[
            matchers.json_params_matcher(
                {"hash": hash, "archive_hash": archive_hash, "size": size}
            )
        ],
        json=model_version_details,
    )

    assert (
        training.create_model_version(api_client, model_id, hash, size, archive_hash)
        == model_version_details
    )


# def test_retrieve_created_model_version(api_client, samples_dir):
#     """There is an existing model version in Created mode, A 400 was raised.
#     But the model is still returned in error content
#     """
#     model_id = "fake_model_id"
#     model_version_id = "fake_model_version_id"
#     # Create a model archive and keep its hash and size.
#     model_file_path = samples_dir / "model_file.pth"
#     training = TrainingMixin()
#     with create_archive(path=model_file_path) as (
#         zst_archive_path,
#         hash,
#         size,
#         archive_hash,
#     ):
#         model_version_details = {
#             "id": model_version_id,
#             "model_id": model_id,
#             "hash": hash,
#             "archive_hash": archive_hash,
#             "size": size,
#             "s3_url": "http://hehehe.com",
#             "s3_put_url": "http://hehehe.com",
#         }

#     responses.add(
#         responses.POST,
#         f"http://testserver/api/v1/model/{model_id}/versions/",
#         status=400,
#         match=[
#             matchers.json_params_matcher(
#                 {"hash": hash, "archive_hash": archive_hash, "size": size}
#             )
#         ],
#         json={"hash": model_version_details},
#     )

#     assert (
#         training.create_model_version(api_client, model_id, hash, size, archive_hash)
#         == model_version_details
#     )


# def test_retrieve_available_model_version(api_client, samples_dir):
#     """Raise error when there is an existing model version in Available mode"""
#     model_id = "fake_model_id"
#     # Create a model archive and keep its hash and size.
#     model_file_path = samples_dir / "model_file.pth"
#     training = TrainingMixin()
#     with create_archive(path=model_file_path) as (
#         zst_archive_path,
#         hash,
#         size,
#         archive_hash,
#     ):
#         responses.add(
#             responses.POST,
#             f"http://testserver/api/v1/model/{model_id}/versions/",
#             status=403,
#             match=[
#                 matchers.json_params_matcher(
#                     {"hash": hash, "archive_hash": archive_hash, "size": size}
#                 )
#             ],
#         )

#     with pytest.raises(Exception):
#         training.create_model_version(api_client, model_id, hash, size, archive_hash)



# def test_handle_s3_uploading_errors(samples_dir):
#     s3_endpoint_url = "http://s3.localhost.com"
#     responses.add_passthru(s3_endpoint_url)
#     responses.add(responses.Response(method="PUT", url=s3_endpoint_url, status=400))
#     file_path = samples_dir / "model_file.pth"
#     training = TrainingMixin()
#     with pytest.raises(Exception):
#         training.upload_to_s3(file_path, {"s3_put_url": s3_endpoint_url})
